@@ -1,33 +1,6 @@
-///////////////////////////////////////////////////////////////////////////
-//
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
-//
-// All rights reserved. This software is distributed under the
-// Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
-//
-// Redistributions of source code must retain the above copyright
-// and license notice and the following restrictions and disclaimer.
-//
-// *     Neither the name of DreamWorks Animation nor the names of
-// its contributors may be used to endorse or promote products derived
-// from this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-// IN NO EVENT SHALL THE COPYRIGHT HOLDERS' AND CONTRIBUTORS' AGGREGATE
-// LIABILITY FOR ALL CLAIMS REGARDLESS OF THEIR BASIS EXCEED US$250.00.
-//
-///////////////////////////////////////////////////////////////////////////
-///
+// Copyright Contributors to the OpenVDB Project
+// SPDX-License-Identifier: MPL-2.0
+
 /// @author Ken Museth
 ///
 /// @file LevelSetPlatonic.h
@@ -41,15 +14,18 @@
 #ifndef OPENVDB_TOOLS_LEVELSETPLATONIC_HAS_BEEN_INCLUDED
 #define OPENVDB_TOOLS_LEVELSETPLATONIC_HAS_BEEN_INCLUDED
 
-#include <vector>
 #include <openvdb/Grid.h>
 #include <openvdb/Types.h>
 #include <openvdb/math/Math.h>
 #include <openvdb/math/Transform.h>
-#include <openvdb/tools/MeshToVolume.h>
 #include <openvdb/util/NullInterrupter.h>
-#include <boost/utility.hpp>
-#include <boost/type_traits/is_floating_point.hpp>
+#include <openvdb/openvdb.h>
+
+#include "MeshToVolume.h"
+
+#include <type_traits>
+#include <vector>
+
 
 namespace openvdb {
 OPENVDB_USE_VERSION_NAMESPACE
@@ -71,12 +47,13 @@ namespace tools {
 /// @note @c GridType::ValueType must be a floating-point scalar.
 template<typename GridType, typename InterruptT>
 typename GridType::Ptr
-createLevelSetPlatonic(int faceCount,// 4, 6, 8, 12 or 20
-                       float scale = 1.0f,
-                       const Vec3f& center = Vec3f(0.0f),
-                       float voxelSize = 0.1f,
-                       float halfWidth = float(LEVEL_SET_HALF_WIDTH),
-                       InterruptT* interrupt = NULL);
+createLevelSetPlatonic(
+    int faceCount, // 4, 6, 8, 12 or 20
+    float scale = 1.0f,
+    const Vec3f& center = Vec3f(0.0f),
+    float voxelSize = 0.1f,
+    float halfWidth = float(LEVEL_SET_HALF_WIDTH),
+    InterruptT* interrupt = nullptr);
 
 /// @brief Return a grid of type @c GridType containing a narrow-band level set
 /// representation of a platonic solid.
@@ -92,11 +69,12 @@ createLevelSetPlatonic(int faceCount,// 4, 6, 8, 12 or 20
 /// @note @c GridType::ValueType must be a floating-point scalar.
 template<typename GridType>
 typename GridType::Ptr
-createLevelSetPlatonic(int faceCount,// 4, 6, 8, 12 or 20
-                       float scale = 1.0f,
-                       const Vec3f& center = Vec3f(0.0f),
-                       float voxelSize = 0.1f,
-                       float halfWidth = float(LEVEL_SET_HALF_WIDTH))
+createLevelSetPlatonic(
+    int faceCount,// 4, 6, 8, 12 or 20
+    float scale = 1.0f,
+    const Vec3f& center = Vec3f(0.0f),
+    float voxelSize = 0.1f,
+    float halfWidth = float(LEVEL_SET_HALF_WIDTH))
 {
     util::NullInterrupter tmp;
     return createLevelSetPlatonic<GridType>(faceCount, scale, center, voxelSize, halfWidth, &tmp);
@@ -116,14 +94,15 @@ createLevelSetPlatonic(int faceCount,// 4, 6, 8, 12 or 20
 /// @note @c GridType::ValueType must be a floating-point scalar.
 template<typename GridType, typename InterruptT>
 typename GridType::Ptr
-createLevelSetTetrahedron(float scale = 1.0f,
-                          const Vec3f& center = Vec3f(0.0f),
-                          float voxelSize = 0.1f,
-                          float halfWidth = float(LEVEL_SET_HALF_WIDTH),
-                          InterruptT* interrupt =  NULL)
+createLevelSetTetrahedron(
+    float scale = 1.0f,
+    const Vec3f& center = Vec3f(0.0f),
+    float voxelSize = 0.1f,
+    float halfWidth = float(LEVEL_SET_HALF_WIDTH),
+    InterruptT* interrupt =  nullptr)
 {
-    return createLevelSetPlatonic<GridType, InterruptT>(4, scale, center,
-                                                        voxelSize, halfWidth, interrupt);
+    return createLevelSetPlatonic<GridType, InterruptT>(
+        4, scale, center, voxelSize, halfWidth, interrupt);
 }
 
 /// @brief Return a grid of type @c GridType containing a narrow-band level set
@@ -137,10 +116,11 @@ createLevelSetTetrahedron(float scale = 1.0f,
 /// @note @c GridType::ValueType must be a floating-point scalar.
 template<typename GridType>
 typename GridType::Ptr
-createLevelSetTetrahedron(float scale = 1.0f,
-                          const Vec3f& center = Vec3f(0.0f),
-                          float voxelSize = 0.1f,
-                          float halfWidth = float(LEVEL_SET_HALF_WIDTH))
+createLevelSetTetrahedron(
+    float scale = 1.0f,
+    const Vec3f& center = Vec3f(0.0f),
+    float voxelSize = 0.1f,
+    float halfWidth = float(LEVEL_SET_HALF_WIDTH))
 {
     util::NullInterrupter tmp;
     return createLevelSetPlatonic<GridType>(4, scale, center, voxelSize, halfWidth, &tmp);
@@ -160,11 +140,12 @@ createLevelSetTetrahedron(float scale = 1.0f,
 /// @note @c GridType::ValueType must be a floating-point scalar.
 template<typename GridType, typename InterruptT>
 typename GridType::Ptr
-createLevelSetCube(float scale = 1.0f,
-                   const Vec3f& center = Vec3f(0.0f),
-                   float voxelSize = 0.1f,
-                   float halfWidth = float(LEVEL_SET_HALF_WIDTH),
-                   InterruptT* interrupt =  NULL)
+createLevelSetCube(
+    float scale = 1.0f,
+    const Vec3f& center = Vec3f(0.0f),
+    float voxelSize = 0.1f,
+    float halfWidth = float(LEVEL_SET_HALF_WIDTH),
+    InterruptT* interrupt =  nullptr)
 {
     return createLevelSetPlatonic<GridType>(6, scale, center, voxelSize, halfWidth, interrupt);
 }
@@ -180,10 +161,11 @@ createLevelSetCube(float scale = 1.0f,
 /// @note @c GridType::ValueType must be a floating-point scalar.
 template<typename GridType>
 typename GridType::Ptr
-createLevelSetCube(float scale = 1.0f,
-                   const Vec3f& center = Vec3f(0.0f),
-                   float voxelSize = 0.1f,
-                   float halfWidth = float(LEVEL_SET_HALF_WIDTH))
+createLevelSetCube(
+    float scale = 1.0f,
+    const Vec3f& center = Vec3f(0.0f),
+    float voxelSize = 0.1f,
+    float halfWidth = float(LEVEL_SET_HALF_WIDTH))
 {
     util::NullInterrupter tmp;
     return createLevelSetPlatonic<GridType>(6, scale, center, voxelSize, halfWidth, &tmp);
@@ -203,11 +185,12 @@ createLevelSetCube(float scale = 1.0f,
 /// @note @c GridType::ValueType must be a floating-point scalar.
 template<typename GridType, typename InterruptT>
 typename GridType::Ptr
-createLevelSetOctahedron(float scale = 1.0f,
-                         const Vec3f& center = Vec3f(0.0f),
-                         float voxelSize = 0.1f,
-                         float halfWidth = float(LEVEL_SET_HALF_WIDTH),
-                         InterruptT* interrupt =  NULL)
+createLevelSetOctahedron(
+    float scale = 1.0f,
+    const Vec3f& center = Vec3f(0.0f),
+    float voxelSize = 0.1f,
+    float halfWidth = float(LEVEL_SET_HALF_WIDTH),
+    InterruptT* interrupt = nullptr)
 {
     return createLevelSetPlatonic<GridType>(8, scale, center, voxelSize, halfWidth, interrupt);
 }
@@ -223,10 +206,11 @@ createLevelSetOctahedron(float scale = 1.0f,
 /// @note @c GridType::ValueType must be a floating-point scalar.
 template<typename GridType>
 typename GridType::Ptr
-createLevelSetOctahedron(float scale = 1.0f,
-                         const Vec3f& center = Vec3f(0.0f),
-                         float voxelSize = 0.1f,
-                         float halfWidth = float(LEVEL_SET_HALF_WIDTH))
+createLevelSetOctahedron(
+    float scale = 1.0f,
+    const Vec3f& center = Vec3f(0.0f),
+    float voxelSize = 0.1f,
+    float halfWidth = float(LEVEL_SET_HALF_WIDTH))
 {
     util::NullInterrupter tmp;
     return createLevelSetPlatonic<GridType>(8, scale, center, voxelSize, halfWidth, &tmp);
@@ -246,11 +230,12 @@ createLevelSetOctahedron(float scale = 1.0f,
 /// @note @c GridType::ValueType must be a floating-point scalar.
 template<typename GridType, typename InterruptT>
 typename GridType::Ptr
-createLevelSetDodecahedron(float scale = 1.0f,
-                           const Vec3f& center = Vec3f(0.0f),
-                           float voxelSize = 0.1f,
-                           float halfWidth = float(LEVEL_SET_HALF_WIDTH),
-                           InterruptT* interrupt =  NULL)
+createLevelSetDodecahedron(
+    float scale = 1.0f,
+    const Vec3f& center = Vec3f(0.0f),
+    float voxelSize = 0.1f,
+    float halfWidth = float(LEVEL_SET_HALF_WIDTH),
+    InterruptT* interrupt = nullptr)
 {
     return createLevelSetPlatonic<GridType>(12, scale, center, voxelSize, halfWidth, interrupt);
 }
@@ -266,10 +251,11 @@ createLevelSetDodecahedron(float scale = 1.0f,
 /// @note @c GridType::ValueType must be a floating-point scalar.
 template<typename GridType>
 typename GridType::Ptr
-createLevelSetDodecahedron(float scale = 1.0f,
-                           const Vec3f& center = Vec3f(0.0f),
-                           float voxelSize = 0.1f,
-                           float halfWidth = float(LEVEL_SET_HALF_WIDTH))
+createLevelSetDodecahedron(
+    float scale = 1.0f,
+    const Vec3f& center = Vec3f(0.0f),
+    float voxelSize = 0.1f,
+    float halfWidth = float(LEVEL_SET_HALF_WIDTH))
 {
     util::NullInterrupter tmp;
     return createLevelSetPlatonic<GridType>(12, scale, center, voxelSize, halfWidth, &tmp);
@@ -289,11 +275,12 @@ createLevelSetDodecahedron(float scale = 1.0f,
 /// @note @c GridType::ValueType must be a floating-point scalar.
 template<typename GridType, typename InterruptT>
 typename GridType::Ptr
-createLevelSetIcosahedron(float scale = 1.0f,
-                          const Vec3f& center = Vec3f(0.0f),
-                          float voxelSize = 0.1f,
-                          float halfWidth = float(LEVEL_SET_HALF_WIDTH),
-                          InterruptT* interrupt =  NULL)
+createLevelSetIcosahedron(
+    float scale = 1.0f,
+    const Vec3f& center = Vec3f(0.0f),
+    float voxelSize = 0.1f,
+    float halfWidth = float(LEVEL_SET_HALF_WIDTH),
+    InterruptT* interrupt = nullptr)
 {
     return createLevelSetPlatonic<GridType>(20, scale, center, voxelSize, halfWidth, interrupt);
 }
@@ -309,10 +296,11 @@ createLevelSetIcosahedron(float scale = 1.0f,
 /// @note @c GridType::ValueType must be a floating-point scalar.
 template<typename GridType>
 typename GridType::Ptr
-createLevelSetIcosahedron(float scale = 1.0f,
-                          const Vec3f& center = Vec3f(0.0f),
-                          float voxelSize = 0.1f,
-                          float halfWidth = float(LEVEL_SET_HALF_WIDTH))
+createLevelSetIcosahedron(
+    float scale = 1.0f,
+    const Vec3f& center = Vec3f(0.0f),
+    float voxelSize = 0.1f,
+    float halfWidth = float(LEVEL_SET_HALF_WIDTH))
 {
     util::NullInterrupter tmp;
     return createLevelSetPlatonic<GridType>(20, scale, center, voxelSize, halfWidth, &tmp);
@@ -323,10 +311,11 @@ createLevelSetIcosahedron(float scale = 1.0f,
 template<typename GridType, typename InterruptT>
 typename GridType::Ptr
 createLevelSetPlatonic(int faceCount,float scale, const Vec3f& center,
-                       float voxelSize, float halfWidth, InterruptT *interrupt)
+    float voxelSize, float halfWidth, InterruptT *interrupt)
 {
     // GridType::ValueType is required to be a floating-point scalar.
-    BOOST_STATIC_ASSERT(boost::is_floating_point<typename GridType::ValueType>::value);
+    static_assert(std::is_floating_point<typename GridType::ValueType>::value,
+        "level set grids must have scalar, floating-point value types");
 
     const math::Transform::Ptr xform = math::Transform::createLinearTransform( voxelSize );
 
@@ -338,7 +327,7 @@ createLevelSetPlatonic(int faceCount,float scale, const Vec3f& center,
 
         vtx.push_back( Vec3f( 0.0f,          1.0f,         0.0f) );
         vtx.push_back( Vec3f(-0.942810297f, -0.333329707f, 0.0f) );
-        vtx.push_back( Vec3f( 0.471405149f, -0.333329707f, 0.816497624) );
+        vtx.push_back( Vec3f( 0.471405149f, -0.333329707f, 0.816497624f) );
         vtx.push_back( Vec3f( 0.471405149f, -0.333329707f, -0.816497624f) );
 
         tri.push_back( Vec3I(0, 2, 3) );
@@ -475,7 +464,7 @@ createLevelSetPlatonic(int faceCount,float scale, const Vec3f& center,
 
     typename GridType::Ptr grid;
 
-    if (interrupt == NULL) {
+    if (interrupt == nullptr) {
         util::NullInterrupter tmp;
         grid = meshToLevelSet<GridType>(tmp, *xform, vtx, tri, qua, halfWidth);
     } else {
@@ -485,12 +474,59 @@ createLevelSetPlatonic(int faceCount,float scale, const Vec3f& center,
     return grid;
 }
 
+
+////////////////////////////////////////
+
+
+// Explicit Template Instantiation
+
+#ifdef OPENVDB_USE_EXPLICIT_INSTANTIATION
+
+#ifdef OPENVDB_INSTANTIATE_LEVELSETPLATONIC
+#include <openvdb/util/ExplicitInstantiation.h>
+#endif
+
+#define _FUNCTION(TreeT) \
+    Grid<TreeT>::Ptr createLevelSetPlatonic<Grid<TreeT>>(int, float, const Vec3f&, float, float, \
+        util::NullInterrupter*)
+OPENVDB_REAL_TREE_INSTANTIATE(_FUNCTION)
+#undef _FUNCTION
+
+#define _FUNCTION(TreeT) \
+    Grid<TreeT>::Ptr createLevelSetTetrahedron<Grid<TreeT>>(float, const Vec3f&, float, float, \
+        util::NullInterrupter*)
+OPENVDB_REAL_TREE_INSTANTIATE(_FUNCTION)
+#undef _FUNCTION
+
+#define _FUNCTION(TreeT) \
+    Grid<TreeT>::Ptr createLevelSetCube<Grid<TreeT>>(float, const Vec3f&, float, float, \
+        util::NullInterrupter*)
+OPENVDB_REAL_TREE_INSTANTIATE(_FUNCTION)
+#undef _FUNCTION
+
+#define _FUNCTION(TreeT) \
+    Grid<TreeT>::Ptr createLevelSetOctahedron<Grid<TreeT>>(float, const Vec3f&, float, float, \
+        util::NullInterrupter*)
+OPENVDB_REAL_TREE_INSTANTIATE(_FUNCTION)
+#undef _FUNCTION
+
+#define _FUNCTION(TreeT) \
+    Grid<TreeT>::Ptr createLevelSetDodecahedron<Grid<TreeT>>(float, const Vec3f&, float, float, \
+        util::NullInterrupter*)
+OPENVDB_REAL_TREE_INSTANTIATE(_FUNCTION)
+#undef _FUNCTION
+
+#define _FUNCTION(TreeT) \
+    Grid<TreeT>::Ptr createLevelSetIcosahedron<Grid<TreeT>>(float, const Vec3f&, float, float, \
+        util::NullInterrupter*)
+OPENVDB_REAL_TREE_INSTANTIATE(_FUNCTION)
+#undef _FUNCTION
+
+#endif // OPENVDB_USE_EXPLICIT_INSTANTIATION
+
+
 } // namespace tools
 } // namespace OPENVDB_VERSION_NAME
 } // namespace openvdb
 
 #endif // OPENVDB_TOOLS_LEVELSETPLATONIC_HAS_BEEN_INCLUDED
-
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
-// All rights reserved. This software is distributed under the
-// Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
