@@ -155,6 +155,7 @@ public:
 
     /// Return the memory in bytes occupied by this node.
     Index64 memUsage() const;
+    Index64 memUsageIfLoaded() const;
 
     /// Expand the given bounding box so that it includes this leaf node's active voxels.
     /// If visitVoxels is false this LeafNode will be approximated as dense, i.e. with all
@@ -182,12 +183,10 @@ public:
     /// Return the global coordinates for a linear table offset.
     Coord offsetToGlobalCoord(Index n) const;
 
-#if OPENVDB_ABI_VERSION_NUMBER >= 9
     /// Return the transient data value.
     Index32 transientData() const { return mTransientData; }
     /// Set the transient data value.
     void setTransientData(Index32 transientData) { mTransientData = transientData; }
-#endif
 
     /// Return a string representation of this node.
     std::string str() const;
@@ -691,25 +690,6 @@ public:
     template<typename CombineOp, typename OtherNodeT /*= LeafNode*/>
     void combine2(const LeafNode& b0, const OtherNodeT& b1, CombineOp&);
 
-    /// @brief Calls the templated functor BBoxOp with bounding box
-    /// information. An additional level argument is provided to the
-    /// callback.
-    ///
-    /// @note The bounding boxes are guaranteed to be non-overlapping.
-    template<typename BBoxOp> void visitActiveBBox(BBoxOp&) const;
-
-    template<typename VisitorOp> void visit(VisitorOp&);
-    template<typename VisitorOp> void visit(VisitorOp&) const;
-
-    template<typename OtherLeafNodeType, typename VisitorOp>
-    void visit2Node(OtherLeafNodeType& other, VisitorOp&);
-    template<typename OtherLeafNodeType, typename VisitorOp>
-    void visit2Node(OtherLeafNodeType& other, VisitorOp&) const;
-    template<typename IterT, typename VisitorOp>
-    void visit2(IterT& otherIter, VisitorOp&, bool otherIsLHS = false);
-    template<typename IterT, typename VisitorOp>
-    void visit2(IterT& otherIter, VisitorOp&, bool otherIsLHS = false) const;
-
     //@{
     /// This function exists only to enable template instantiation.
     void prune(const ValueType& /*tolerance*/ = zeroVal<ValueType>()) {}
@@ -888,17 +868,6 @@ protected:
     /// Compute the origin of the leaf node that contains the voxel with the given coordinates.
     static void evalNodeOrigin(Coord& xyz) { xyz &= ~(DIM - 1); }
 
-    template<typename NodeT, typename VisitorOp, typename ChildAllIterT>
-    static inline void doVisit(NodeT&, VisitorOp&);
-
-    template<typename NodeT, typename OtherNodeT, typename VisitorOp,
-             typename ChildAllIterT, typename OtherChildAllIterT>
-    static inline void doVisit2Node(NodeT& self, OtherNodeT& other, VisitorOp&);
-
-    template<typename NodeT, typename VisitorOp,
-             typename ChildAllIterT, typename OtherChildAllIterT>
-    static inline void doVisit2(NodeT& self, OtherChildAllIterT&, VisitorOp&, bool otherIsLHS);
-
 private:
     /// Buffer containing the actual data values
     Buffer mBuffer;
@@ -906,10 +875,8 @@ private:
     NodeMaskType mValueMask;
     /// Global grid index coordinates (x,y,z) of the local origin of this node
     Coord mOrigin;
-#if OPENVDB_ABI_VERSION_NUMBER >= 9
     /// Transient data (not serialized)
     Index32 mTransientData = 0;
-#endif
 }; // end of LeafNode class
 
 
@@ -965,9 +932,7 @@ LeafNode<T, Log2Dim>::LeafNode(const LeafNode& other)
     : mBuffer(other.mBuffer)
     , mValueMask(other.valueMask())
     , mOrigin(other.mOrigin)
-#if OPENVDB_ABI_VERSION_NUMBER >= 9
     , mTransientData(other.mTransientData)
-#endif
 {
 }
 
@@ -979,9 +944,7 @@ inline
 LeafNode<T, Log2Dim>::LeafNode(const LeafNode<OtherValueType, Log2Dim>& other)
     : mValueMask(other.valueMask())
     , mOrigin(other.mOrigin)
-#if OPENVDB_ABI_VERSION_NUMBER >= 9
     , mTransientData(other.mTransientData)
-#endif
 {
     struct Local {
         /// @todo Consider using a value conversion functor passed as an argument instead.
@@ -1002,9 +965,7 @@ LeafNode<T, Log2Dim>::LeafNode(const LeafNode<OtherValueType, Log2Dim>& other,
     : mBuffer(background)
     , mValueMask(other.valueMask())
     , mOrigin(other.mOrigin)
-#if OPENVDB_ABI_VERSION_NUMBER >= 9
     , mTransientData(other.mTransientData)
-#endif
 {
 }
 
@@ -1016,9 +977,7 @@ LeafNode<T, Log2Dim>::LeafNode(const LeafNode<OtherValueType, Log2Dim>& other,
     const ValueType& offValue, const ValueType& onValue, TopologyCopy)
     : mValueMask(other.valueMask())
     , mOrigin(other.mOrigin)
-#if OPENVDB_ABI_VERSION_NUMBER >= 9
     , mTransientData(other.mTransientData)
-#endif
 {
     for (Index i = 0; i < SIZE; ++i) {
         mBuffer[i] = (mValueMask.isOn(i) ? onValue : offValue);
@@ -1355,7 +1314,9 @@ LeafNode<T,Log2Dim>::readBuffers(std::istream& is, const CoordBBox& clipBBox, bo
     SharedPtr<io::StreamMetadata> meta = io::getStreamMetadataPtr(is);
     const bool seekable = meta && meta->seekable();
 
+#ifdef OPENVDB_USE_DELAYED_LOADING
     std::streamoff maskpos = is.tellg();
+#endif
 
     if (seekable) {
         // Seek over the value mask.
@@ -1381,6 +1342,7 @@ LeafNode<T,Log2Dim>::readBuffers(std::istream& is, const CoordBBox& clipBBox, bo
         mValueMask.setOff();
         mBuffer.setOutOfCore(false);
     } else {
+#ifdef OPENVDB_USE_DELAYED_LOADING
         // If this node lies completely inside the clipping region and it is being read
         // from a memory-mapped file, delay loading of its buffer until the buffer
         // is actually accessed.  (If this node requires clipping, its buffer
@@ -1400,6 +1362,7 @@ LeafNode<T,Log2Dim>::readBuffers(std::istream& is, const CoordBBox& clipBBox, bo
             // Skip over voxel values.
             skipCompressedValues(seekable, is, fromHalf);
         } else {
+#endif
             mBuffer.allocate();
             io::readCompressedValues(is, mBuffer.mData, SIZE, mValueMask, fromHalf);
             mBuffer.setOutOfCore(false);
@@ -1410,7 +1373,9 @@ LeafNode<T,Log2Dim>::readBuffers(std::istream& is, const CoordBBox& clipBBox, bo
                 background = *static_cast<const T*>(bgPtr);
             }
             this->clip(clipBBox, background);
+#ifdef OPENVDB_USE_DELAYED_LOADING
         }
+#endif
     }
 
     if (numBuffers > 1) {
@@ -1469,6 +1434,16 @@ LeafNode<T, Log2Dim>::memUsage() const
 }
 
 
+template<typename T, Index Log2Dim>
+inline Index64
+LeafNode<T, Log2Dim>::memUsageIfLoaded() const
+{
+    // Use sizeof(*this) to capture alignment-related padding
+    // (but note that sizeof(*this) includes sizeof(mBuffer)).
+    return sizeof(*this) + mBuffer.memUsageIfLoaded() - sizeof(mBuffer);
+}
+
+
 template<typename T, Index Log2Dim>
 inline void
 LeafNode<T, Log2Dim>::evalActiveBoundingBox(CoordBBox& bbox, bool visitVoxels) const
@@ -1840,148 +1815,6 @@ LeafNode<T, Log2Dim>::combine2(const LeafNode& b0, const OtherNodeT& b1, Combine
 ////////////////////////////////////////
 
 
-template<typename T, Index Log2Dim>
-template<typename BBoxOp>
-inline void
-LeafNode<T, Log2Dim>::visitActiveBBox(BBoxOp& op) const
-{
-    if (op.template descent<LEVEL>()) {
-        for (ValueOnCIter i=this->cbeginValueOn(); i; ++i) {
-            op.template operator()<LEVEL>(CoordBBox::createCube(i.getCoord(), 1));
-        }
-    } else {
-        op.template operator()<LEVEL>(this->getNodeBoundingBox());
-    }
-}
-
-
-template<typename T, Index Log2Dim>
-template<typename VisitorOp>
-inline void
-LeafNode<T, Log2Dim>::visit(VisitorOp& op)
-{
-    doVisit<LeafNode, VisitorOp, ChildAllIter>(*this, op);
-}
-
-
-template<typename T, Index Log2Dim>
-template<typename VisitorOp>
-inline void
-LeafNode<T, Log2Dim>::visit(VisitorOp& op) const
-{
-    doVisit<const LeafNode, VisitorOp, ChildAllCIter>(*this, op);
-}
-
-
-template<typename T, Index Log2Dim>
-template<typename NodeT, typename VisitorOp, typename ChildAllIterT>
-inline void
-LeafNode<T, Log2Dim>::doVisit(NodeT& self, VisitorOp& op)
-{
-    for (ChildAllIterT iter = self.beginChildAll(); iter; ++iter) {
-        op(iter);
-    }
-}
-
-
-////////////////////////////////////////
-
-
-template<typename T, Index Log2Dim>
-template<typename OtherLeafNodeType, typename VisitorOp>
-inline void
-LeafNode<T, Log2Dim>::visit2Node(OtherLeafNodeType& other, VisitorOp& op)
-{
-    doVisit2Node<LeafNode, OtherLeafNodeType, VisitorOp, ChildAllIter,
-        typename OtherLeafNodeType::ChildAllIter>(*this, other, op);
-}
-
-
-template<typename T, Index Log2Dim>
-template<typename OtherLeafNodeType, typename VisitorOp>
-inline void
-LeafNode<T, Log2Dim>::visit2Node(OtherLeafNodeType& other, VisitorOp& op) const
-{
-    doVisit2Node<const LeafNode, OtherLeafNodeType, VisitorOp, ChildAllCIter,
-        typename OtherLeafNodeType::ChildAllCIter>(*this, other, op);
-}
-
-
-template<typename T, Index Log2Dim>
-template<
-    typename NodeT,
-    typename OtherNodeT,
-    typename VisitorOp,
-    typename ChildAllIterT,
-    typename OtherChildAllIterT>
-inline void
-LeafNode<T, Log2Dim>::doVisit2Node(NodeT& self, OtherNodeT& other, VisitorOp& op)
-{
-    // Allow the two nodes to have different ValueTypes, but not different dimensions.
-    static_assert(OtherNodeT::SIZE == NodeT::SIZE,
-        "can't visit nodes of different sizes simultaneously");
-    static_assert(OtherNodeT::LEVEL == NodeT::LEVEL,
-        "can't visit nodes at different tree levels simultaneously");
-
-    ChildAllIterT iter = self.beginChildAll();
-    OtherChildAllIterT otherIter = other.beginChildAll();
-
-    for ( ; iter && otherIter; ++iter, ++otherIter) {
-        op(iter, otherIter);
-    }
-}
-
-
-////////////////////////////////////////
-
-
-template<typename T, Index Log2Dim>
-template<typename IterT, typename VisitorOp>
-inline void
-LeafNode<T, Log2Dim>::visit2(IterT& otherIter, VisitorOp& op, bool otherIsLHS)
-{
-    doVisit2<LeafNode, VisitorOp, ChildAllIter, IterT>(
-        *this, otherIter, op, otherIsLHS);
-}
-
-
-template<typename T, Index Log2Dim>
-template<typename IterT, typename VisitorOp>
-inline void
-LeafNode<T, Log2Dim>::visit2(IterT& otherIter, VisitorOp& op, bool otherIsLHS) const
-{
-    doVisit2<const LeafNode, VisitorOp, ChildAllCIter, IterT>(
-        *this, otherIter, op, otherIsLHS);
-}
-
-
-template<typename T, Index Log2Dim>
-template<
-    typename NodeT,
-    typename VisitorOp,
-    typename ChildAllIterT,
-    typename OtherChildAllIterT>
-inline void
-LeafNode<T, Log2Dim>::doVisit2(NodeT& self, OtherChildAllIterT& otherIter,
-    VisitorOp& op, bool otherIsLHS)
-{
-    if (!otherIter) return;
-
-    if (otherIsLHS) {
-        for (ChildAllIterT iter = self.beginChildAll(); iter; ++iter) {
-            op(otherIter, iter);
-        }
-    } else {
-        for (ChildAllIterT iter = self.beginChildAll(); iter; ++iter) {
-            op(iter, otherIter);
-        }
-    }
-}
-
-
-////////////////////////////////////////
-
-
 template<typename T, Index Log2Dim>
 inline std::ostream&
 operator<<(std::ostream& os, const typename LeafNode<T, Log2Dim>::Buffer& buf)
